@@ -25,6 +25,8 @@
 #include <config.h>
 #endif
 
+#include <glib/gi18n-lib.h>
+
 #include <libedata-book/libedata-book.h>
 
 #include "server/e-source-ews-folder.h"
@@ -66,6 +68,9 @@ e_book_backend_ews_factory_init (EBookBackendFactory *factory)
 G_MODULE_EXPORT void
 e_module_load (GTypeModule *type_module)
 {
+	bindtextdomain (GETTEXT_PACKAGE, EXCHANGE_EWS_LOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
 	e_source_ews_folder_type_register (type_module);
 
 	e_book_backend_ews_factory_register_type (type_module);
