@@ -10,6 +10,7 @@
 #endif
 
 #include <string.h>
+#include <glib/gi18n-lib.h>
 
 #include <libedata-cal/libedata-cal.h>
 
@@ -109,6 +110,9 @@ e_cal_backend_ews_todos_factory_init (ECalBackendFactory *factory)
 G_MODULE_EXPORT void
 e_module_load (GTypeModule *type_module)
 {
+	bindtextdomain (GETTEXT_PACKAGE, EXCHANGE_EWS_LOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
 	e_source_ews_folder_type_register (type_module);
 
 	e_cal_backend_ews_events_factory_register_type (type_module);
