@@ -31,6 +31,7 @@
 #include "e-mail-config-ews-oal-combo-box.h"
 #include "e-mail-config-ews-delegates-page.h"
 #include "e-mail-config-ews-ooo-page.h"
+#include "e-mail-config-ews-folder-sizes-page.h"
 #include "e-ews-ooo-notificator.h"
 
 #include "e-ews-config-ui-extension.h"
@@ -55,6 +56,7 @@ e_module_load (GTypeModule *type_module)
 	e_mail_config_ews_oal_combo_box_type_register (type_module);
 	e_mail_config_ews_delegates_page_type_register (type_module);
 	e_mail_config_ews_ooo_page_type_register (type_module);
+	e_mail_config_ews_folder_sizes_page_type_register (type_module);
 	e_ews_config_ui_extension_type_register (type_module);
 	e_ews_ooo_notificator_type_register (type_module);
 
