@@ -35,6 +35,7 @@ struct _CamelEwsSettingsPrivate {
 	gboolean filter_junk_inbox;
 	gboolean oab_offline;
 	gboolean use_impersonation;
+	gboolean folders_initialized;
 	gchar *email;
 	gchar *gal_uid;
 	gchar *hosturl;
@@ -51,6 +52,7 @@ enum {
 	PROP_EMAIL,
 	PROP_FILTER_JUNK,
 	PROP_FILTER_JUNK_INBOX,
+	PROP_FOLDERS_INITIALIZED,
 	PROP_GAL_UID,
 	PROP_HOST,
 	PROP_HOSTURL,
@@ -109,6 +111,12 @@ ews_settings_set_property (GObject *object,
 				g_value_get_boolean (value));
 			return;
 
+		case PROP_FOLDERS_INITIALIZED:
+			camel_ews_settings_set_folders_initialized (
+				CAMEL_EWS_SETTINGS (object),
+				g_value_get_boolean (value));
+			return;
+
 		case PROP_GAL_UID:
 			camel_ews_settings_set_gal_uid (
 				CAMEL_EWS_SETTINGS (object),
@@ -227,6 +235,13 @@ ews_settings_get_property (GObject *object,
 				CAMEL_EWS_SETTINGS (object)));
 			return;
 
+		case PROP_FOLDERS_INITIALIZED:
+			g_value_set_boolean (
+				value,
+				camel_ews_settings_get_folders_initialized (
+				CAMEL_EWS_SETTINGS (object)));
+			return;
+
 		case PROP_GAL_UID:
 			g_value_take_string (
 				value,
@@ -401,6 +416,18 @@ camel_ews_settings_class_init (CamelEwsSettingsClass *class)
 			G_PARAM_CONSTRUCT |
 			G_PARAM_STATIC_STRINGS));
 
+	g_object_class_install_property (
+		object_class,
+		PROP_FOLDERS_INITIALIZED,
+		g_param_spec_boolean (
+			"folders-initialized",
+			"Folders Initialized",
+			"Whether Sent Items and Drafts folders are initialized",
+			FALSE,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT |
+			G_PARAM_STATIC_STRINGS));
+
 	g_object_class_install_property (
 		object_class,
 		PROP_GAL_UID,
@@ -570,6 +597,50 @@ camel_ews_settings_set_check_all (CamelEwsSettings *settings,
 	g_object_notify (G_OBJECT (settings), "check-all");
 }
 
+/**
+ * camel_ews_settings_get_folders_initialized:
+ * @settings: a #CamelEwsSettings
+ *
+ * Returns whether folders had been initialized.
+ * This is used to know whether it's required to setup Drafts
+ * and Sent Items folders as folders for that purpose in Evolution.
+ *
+ * Returns: whether folders had been initialized
+ *
+ * Since: 3.6
+ **/
+gboolean
+camel_ews_settings_get_folders_initialized (CamelEwsSettings *settings)
+{
+	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), FALSE);
+
+	return settings->priv->folders_initialized;
+}
+
+/**
+ * camel_ews_settings_set_folders_initialized:
+ * @settings: a #CamelEwsSettings
+ * @folders_initialized: whether folders had been initialized
+ *
+ * Sets whether to folders had been initialized. This is for Drafts
+ * and Sent Items folders only.
+ *
+ * Since: 3.6
+ **/
+void
+camel_ews_settings_set_folders_initialized (CamelEwsSettings *settings,
+					    gboolean folders_initialized)
+{
+	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
+
+	if ((settings->priv->folders_initialized ? 1 : 0) == (folders_initialized ? 1 : 0))
+		return;
+
+	settings->priv->folders_initialized = folders_initialized;
+
+	g_object_notify (G_OBJECT (settings), "folders-initialized");
+}
+
 const gchar *
 camel_ews_settings_get_email (CamelEwsSettings *settings)
 {
