@@ -105,8 +105,6 @@ ews_oof_settings_text_from_html (gchar *html_text)
 	gchar *start, *end;
 	gchar *ii, *jj;
 
-	/* Note, this consumes the input string. */
-
 	g_return_val_if_fail (html_text != NULL, NULL);
 
 	haystack_len = strlen (html_text);
@@ -131,8 +129,6 @@ ews_oof_settings_text_from_html (gchar *html_text)
 	}
 	*jj = '\0';
 
-	g_free (html_text);
-
 	return plain_text;
 }
 
@@ -247,7 +243,7 @@ ews_oof_settings_get_response_cb (ESoapResponse *response,
 	else if (g_strrstr (string, "BodyFragment") != NULL)
 		text = ews_oof_settings_text_from_html (string);
 	else
-		text = NULL;
+		text = g_strdup (string);
 	e_ews_oof_settings_set_internal_reply (settings, text ? text : "");
 	g_free (string);
 	g_free (text);
@@ -264,7 +260,7 @@ ews_oof_settings_get_response_cb (ESoapResponse *response,
 	else if (g_strrstr (string, "BodyFragment") != NULL)
 		text = ews_oof_settings_text_from_html (string);
 	else
-		text = NULL;
+		text = g_strdup (string);
 	e_ews_oof_settings_set_external_reply (settings, text ? text : "");
 	g_free (string);
 	g_free (text);
